(*                                                                           *)
(*   Quantum optics library: single mode electromagnetic field.              *)
(*                           ([Part of] Layer 3)                             *)
(*                                                                           *)
(*    (c) Copyright, Mohamed Yousri Mahmoud 2013-2014						 *)
(*                   Hardware Verification Group,                            *)
(*                   Concordia University                                    *)
(*                                                                           *)
(*           Contact: <mosolim@ece.concordia.ca>,							 *)
(*                                                                           *)
(*                                                                           *)
(* ========================================================================= *)






let tensor = new_recursive_definition num_RECURSION
   `tensor 0 (modes:bqs^N) = K (Cx(&1)) /\
    tensor (SUC n) (modes) = (\y:real^N. ((tensor n modes) y) *  (modes $ (SUC n)) (y $ (SUC n)))`;;
	
let is_tensor = new_definition
  `is_tensor (tens:qop^N->(real^N->complex)-> (real^N->complex)) <=>
    !(ops:qop^N). is_linear_cop (tens ops) /\ 
    ( !(modes:bqs^N) n. (! i. 1<= i /\ i<= dimindex(:N) ==>  
	modes$i IN sq_integrable /\ (ops$i) (modes$i) IN sq_integrable) ==> 
	  tens ops  (tensor n modes) = tensor n (lambda i. (ops$i) (modes$i)))`;;

let COP_TENSOR_CFUN = prove
(`!(tens:qop^N->(real^N->complex)-> (real^N->complex)) ops modes n. is_tensor tens ==>
 (! i. 1<= i /\ i<= dimindex(:N) ==>  
	modes$i IN sq_integrable /\ (ops$i) (modes$i) IN sq_integrable) ==>  
    tens ops  (tensor n modes) = tensor n (lambda i. (ops$i) (modes$i))`,
   SIMP_TAC[is_tensor]);; 
   
let COP_TENSOR_LINEAR = prove
(`!(tens:qop^N->(real^N->complex)-> (real^N->complex)) ops. is_tensor tens ==>  is_linear_cop (tens ops)`,
   SIMP_TAC[is_tensor]);; 
 
let CFUN_LAMBDA_APP = prove(`!f g.((lambda i. f i (g i)):cfun^N) =
   (lambda i. (((lambda j. f j):(cfun->cfun)^N)$i) (((lambda j. g j):cfun^N)$i))`
   ,SIMP_TAC[CART_EQ;LAMBDA_BETA]);;
 
let COP_LAMBDA_COMPS = prove(
`! f g x. (lambda i. ((((lambda j. f j):(B->A)^N)$i) o
   (((lambda j. g j):(C->B)^N)$i)) ):(C->A)^N = (lambda i. (((lambda j. (f j) o (g j)):(C->A)^N)$i) ):(C->A)^N`,
   SIMP_TAC[o_THM;CART_EQ;LAMBDA_BETA]);;
   
let LAMBDA_RAPP = prove(`!f g.((lambda i. (f i) (((lambda j. g j):B^N)$i)):A^N) =
   (lambda i. (f i) (g i))`
   ,SIMP_TAC[CART_EQ;LAMBDA_BETA]);;

let LAMBDA_LAPP = prove(`!f g.((lambda i.  (((lambda j. g j):(B->A)^N)$i) (f i)):A^N) =
   (lambda i. (g i) (f i) )`
   ,SIMP_TAC[CART_EQ;LAMBDA_BETA]);; 
   
let LAMBDA_BETA_IF =    prove(`!f g a p.((lambda i.  if p i then a%(((lambda j. f j):cfun^N)$i)
 else  (((lambda j. g j):cfun^N)$i)):cfun^N) =  (lambda i. if p i then a%(f i) else (g i))`,
   SIMP_TAC[CART_EQ;LAMBDA_BETA]);; 
   
let LAMBDA_BETA_IF =    prove(`!f g p.((lambda i.  if p i then f i
 else  (((lambda j. g j):cfun^N)$i)):cfun^N) =  (lambda i. if p i then (f i) else (g i))`,
   SIMP_TAC[CART_EQ;LAMBDA_BETA]);; 

let LAMBDA_BETA_IF =    prove(`!f g a p.((lambda i. if p i then a%(((lambda j. f j):cfun^N)$i)
 else  g i):cfun^N) =  (lambda i. if p i then a%(f i) else (g i))`,
   SIMP_TAC[CART_EQ;LAMBDA_BETA]);; 
   

let COP_TENSOR_ASSOC= prove
 (`!ten (ops1:qop^N) (ops2:qop^N) n (modes:bqs^N).
     is_tensor ten /\
	 (! i. 1<= i /\ i<= dimindex(:N) ==>  
	modes$i IN sq_integrable /\ (ops1$i) (modes$i) IN sq_integrable /\ (ops2$i) ((ops1$i) (modes$i)) IN sq_integrable)
	 ==> ten ops2 ( ten ops1 (tensor n modes)) =
	 ten ((lambda i. (ops2$i) o (ops1$i)):qop^N) (tensor n modes)`,
	 SIMP_TAC[is_tensor;LAMBDA_RAPP;LAMBDA_LAPP;o_DEF;CART_EQ;LAMBDA_BETA]);;
	 
	 (****************************************************************************)
(* BEAM SPLITTER                                                            *)
(****************************************************************************)

new_type_abbrev("bmsp",`:complex#complex#complex#complex#sm#sm#sm#sm`);;


let pos = new_definition
 `pos (tens:qop^N->(real^N->complex)-> (real^N->complex)) (op:qop) m = 
    tens (lambda i. if i = m then op else I)`;;
	

let is_beam_splitter = new_definition 
  `is_beam_splitter (p1,p2,p3,p4,ten,i1,m1,i2,m2,o1,m3,o2,m4) <=> 
     is_sm6 i1 /\ is_sm6 i2 /\ is_sm6 o1 /\ is_sm6 o2
     /\ w1 i1 = w1 i2 /\ w1 i2 = w1 o1 /\ w1 o1 = w1 o2 /\ 
       vac i1 = vac i2 /\ vac i2 = vac o1 /\ vac o1 = vac o2 /\  
     pos ten (anh i1) m1 = p1 % pos ten (anh o1) m3 + 
       p2 % pos ten (anh o2) m4 
    /\ pos ten (anh i2 ) m2  = p3 % pos ten (anh o1) m3 + 
       p4 % pos ten (anh o2) m4 
    /\
	pos ten (cr i1) m1  = cnj p1 % pos ten (cr o1) m3  + 
       cnj p2 % pos ten (cr o2)  m4 
    /\ pos ten (cr i2) m2  = cnj p3 % pos ten (cr o1) m3  + 
       cnj p4 % pos ten (cr o2) m4`;;
	   
(****************************************************************************)
(* MACH-ZEHNDER                                                             *)
(****************************************************************************)

let keylem  =   GEN_ALL ( MESON[]  `(p x ==> f x = q) ==> (if p x then q else (f x)) = f x`);;
let MULTI_MODE_DECOMPOSE_TAC = 
        ONCE_REWRITE_TAC[MESON[I_THM] `(if p then (x:bqs) else y) = (if p then x else I y)`] THEN 
        ONCE_REWRITE_TAC[MESON[] `(if p then f1 x else f2 y) = (if p then f1  else f2 ) (if p then  x else  y)`]
        THEN ONCE_REWRITE_TAC[CFUN_LAMBDA_APP] THEN
        SIMP_TAC[keylem;ARITH;COND_ID] THEN
        FIRST_ASSUM(fun th -> IMP_REWRITE_TAC[(MATCH_MP (GSYM COP_TENSOR_CFUN) th)]);;
		

let CFUN_FLATTEN_TAC =
            REWRITE_TAC[COP_MUL_THM;COP_SMUL_THM;COP_ADD_THM;CFUN_SMUL;FUN_EQ_THM;CFUN_ADD_THM]
            THEN REWRITE_TAC[COMPLEX_MUL_SYM;COMPLEX_ADD_AC];;
let mirror = new_definition
	   `mirror (ten,i1,m1,o1,m2) <=> 
   pos ten (cr i1) m1 = --ii % pos ten (cr o1) m2 /\
   pos ten (anh i1) m1 = ii % pos ten (anh o1) m2`;;
   
  
let CR_VAC = prove
(`!sm. is_sm6 sm ==>  ((cr sm) (vac sm)) IN sq_integrable`,
  REPEAT STRIP_TAC THEN FIRST_ASSUM(MP_TAC o (SPEC `SUC 0`) o(MATCH_MP CR_INDH))
  THEN SIMP_TAC [cop_pow;dh;danh;IN_ELIM_THM;COP_MUL_RID]);;
let MACH_ZENDER_INTERFEROMETER = prove
  (`!a b c (ten:qop^N->(real^N->complex)-> (real^N->complex)).
	   is_tensor ten /\
	   is_beam_splitter (--Cx(sqrt(&1/ &2))*ii,Cx(sqrt(&1/ &2)),
           --Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2))*ii ,ten,a$1,1,a$2,2,b$1,1,b$2,2)/\
	   mirror (ten,b$1,1,b$3,1) /\ mirror (ten,b$2,2,b$4,2) /\
	   is_beam_splitter (--Cx(sqrt(&1/ &2))*ii,Cx(sqrt(&1/ &2)),
	   --Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2))*ii,ten,b$3,1,b$4,2,c$1,1,c$2,2)
	   /\  vac (b$2) = vac (c$2) ==>
		tensor 2 
		((lambda i. if i = 1 then fock  (a$1) 1  else vac (a$1)):bqs^N) =
		ii % tensor 2 ((lambda i. if i = 1 then  fock  (c$1) 1  else vac (c$1)):bqs^N)`,
	REWRITE_TAC[is_beam_splitter;mirror;pos] THEN REPEAT STRIP_TAC
	THEN ASM_SIMP_TAC[LET_RULE_L[GSYM COP_SMUL_THM] FOCK_HERM_VAC]  THEN 
	MULTI_MODE_DECOMPOSE_TAC
	THEN SIMP_TAC[cop_pow;COP_MUL_RID;ONE;FACT] THEN
        SIMP_TAC[GSYM ONE] THEN SIMP_TAC[ MULT_CLAUSES;SQRT_1;COMPLEX_INV_1;
        COP_SMUL_LID] THEN
	ASM_SIMP_TAC[CNJ_CX;GSYM CX_NEG;COP_ADD_LDISTRIB] THEN
        REWRITE_TAC[COP_SMUL_ASSOC;CNJ_MUL;CNJ_NEG;CNJ_II;CNJ_CX] 
	THEN REWRITE_TAC[CX_DEF;ii;complex_mul;RE;IM;RE_NEG;IM_NEG;REAL_MUL_LZERO;
        REAL_MUL_RZERO;REAL_NEG_0;REAL_SUB_LZERO;REAL_SUB_RZERO;REAL_ADD_RID;REAL_ADD_LID;
        REAL_MUL_RNEG;REAL_MUL_LNEG;REAL_NEG_NEG;REAL_MUL_RID]
        THEN IMP_REWRITE_TAC[REWRITE_RULE[REAL_POW_2]SQRT_POW_2] THEN REPEAT 
        STRIP_TAC THEN (REAL_ARITH_TAC ORELSE COP_ARITH_TAC ORELSE 
		( ASM_SIMP_TAC[VAC_QST_PROP;LAMBDA_LAPP;LAMBDA_BETA;CR_VAC;I_THM] THEN 
		COND_CASES_TAC THEN ASM_SIMP_TAC[VAC_OTHER_CR_1;FOCK_IN_S;I_THM;VAC_QST_PROP])));;

(****************************************************************************)
(* CNOT GATE                                                                *)
(****************************************************************************)
		
		let DIMINDEX_6 = MATCH_MP DIMINDEX_UNIQUE (define_finite_type 6);;

let ALL_CNOT_TAC = REWRITE_TAC[is_beam_splitter;pos] THEN REPEAT STRIP_TAC
           THEN ASM_SIMP_TAC[LET_RULE_L[GSYM COP_SMUL_THM] FOCK_HERM_VAC]
	   THEN MULTI_MODE_DECOMPOSE_TAC THEN MULTI_MODE_DECOMPOSE_TAC
	   THEN SIMP_TAC[cop_pow;COP_MUL_RID;ONE;FACT] THEN
	   SIMP_TAC[GSYM ONE] THEN
	   SIMP_TAC[ MULT_CLAUSES;SQRT_1;COMPLEX_INV_1;COP_SMUL_LID] THEN
	   ASM_SIMP_TAC[CNJ_CX;GSYM CX_NEG;COP_ADD_LDISTRIB;GSYM COP_MUL_THM] THEN
	   IMP_REWRITE_TAC[LINCOP_ADD_MUL_LDISTRIB;LINCOP_MUL_RMUL;COP_ADD_MUL_RDISTRIB]
	   THEN ASM_SIMP_TAC[CNJ_MUL;COP_ADD_LDISTRIB;COP_SMUL_ASSOC;COP_MUL_LSMUL;GSYM CX_MUL;
           COP_TENSOR_LINEAR;SMUL_LINCOP;ADD_LINCOP;CNJ_CX;CNJ_II;GSYM CX_NEG]
	   THEN REWRITE_TAC[REAL_MUL_RNEG;REAL_MUL_LNEG; REAL_NEG_NEG;CX_NEG;COMPLEX_MUL_ASSOC]
           THEN IMP_REWRITE_TAC[GSYM SQRT_MUL] THEN REPEAT STRIP_TAC THEN (REAL_ARITH_TAC ORELSE ALL_TAC) THEN
            REWRITE_TAC[real_div;GSYM REAL_MUL_ASSOC;REAL_MUL_RID;REAL_MUL_LID;GSYM REAL_INV_MUL;REAL_OF_NUM_MUL;
	    ARITH;REAL_FIELD`inv (&2) * &2 * x= x /\ &2 * inv (&2) * x = x /\  &2 * inv (&6) = inv(&3) /\ 
            inv(&3) * &2 * inv (&3) = &2 * inv(&9)`;COP_ADD_ASSOC] THEN 
           CFUN_FLATTEN_TAC THEN 
            REWRITE_TAC[GSYM pos;GSYM COMPLEX_ADD_LDISTRIB; COMPLEX_FIELD `!a b c. a*b+a*c+d = (a*b+a*c)+d `;
	     GSYM complex_sub;COMPLEX_SUB_REFL;COMPLEX_MUL_RZERO;COMPLEX_ADD_LID]
	    THEN SIMP_TAC[SQRT_INV;REAL_FIELD `&9 = &3 pow 2 /\ &36 = &6 pow 2 /\ &18 = &9 * &2/\ &2 * inv(&6) = inv(&3)`
           ;POW_2_SQRT;REAL_POS;SQRT_MUL;GSYM CX_ADD;GSYM REAL_MUL_2]
		   THEN REPEAT(ASM_SIMP_TAC[VAC_QST_PROP;LAMBDA_LAPP;LAMBDA_BETA;CR_VAC;I_THM] THEN 
		COND_CASES_TAC THEN ASM_SIMP_TAC[VAC_OTHER_CR_1;FOCK_IN_S;I_THM;
		CREAT_FOCK;VAC_QST_PROP;tem INNER_SPACE_SMUL])
		   ;;

let CNOT_C0_T0 = prove
 (`!(ten:qop^6->(real^6->complex)-> (real^6->complex)) (a:sm^6) (b:sm^6) (c:sm^6) (d:sm^6).
	    is_tensor ten /\
		
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,a$2,2,a$1,1,b$2,2,b$1,1)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,a$4,4,a$5,5,b$4,4,b$5,5)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$4,4,a$3,3,c$4,4,c$3,3)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$5,5,a$6,6,c$5,5,c$6,6)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,c$4,4,c$5,5,d$4,4,d$5,5)
         /\ vac (c$6)	= vac (b$1)   
	 ==>
	   (tensor 6 ((lambda i. if i = 2 then fock (a$2) 1 else (if i = 4 then fock (a$4) 1 else vac (a$2))):bqs^6)) =
	      (Cx (inv (&3)) %
	      pos ten (cr (b$2)) 2 ** pos ten (cr (d$4)) 4 + Cx (inv (&3)) %
	      pos ten (cr (b$2)) 2 ** pos ten (cr (c$6)) 6 + Cx (inv (&3)) %
	      pos ten (cr (b$2)) 2 ** pos ten (cr (c$3)) 3 + Cx (&2 * inv (&3 * sqrt (&2))) %
	      pos ten (cr (b$1)) 1 ** pos ten (cr (d$4)) 4 + Cx (sqrt (&2) * inv (&3)) %
	      pos ten (cr (b$1)) 1 ** pos ten (cr (c$6)) 6  + Cx (sqrt (&2) * inv (&3)) % 
	      pos ten (cr (b$1)) 1 ** pos ten (cr (c$3)) 3)  (tensor 6 (lambda j. vac (b$1)))`,	
			
	   ALL_CNOT_TAC);;

let CNOT_C0_T1 = prove
 (`!a b c d (ten:qop^N->(real^N->complex)-> (real^N->complex)).
	    is_tensor ten /\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,a$2,2,a$1,1,b$2,2,b$1,1)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,a$4,4,a$5,5,b$4,4,b$5,5)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$4,4,a$3,3,c$4,4,c$3,3)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$5,5,a$6,6,c$5,5,c$6,6)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,c$4,4,c$5,5,d$4,4,d$5,5)
         /\ vac (c$6)	= vac (b$1)   
	 ==>
	   (tensor 6 ((lambda i. if i = 2 then fock (a$2) 1 else (if i = 5 then fock (a$5) 1 else vac (a$2))):bqs^N)) =
	(--Cx (inv (&3))  % 
         pos ten (cr (b$2)) 2 ** pos ten (cr (c$6)) 6 + Cx (inv (&3)) % 
	 pos ten (cr (b$2)) 2 ** pos ten (cr (d$5)) 5 + Cx (inv (&3)) %
         pos ten (cr (b$2)) 2 ** pos ten (cr (c$3)) 3 + --Cx (sqrt (&2) * inv (&3)) %
     	 pos ten (cr (b$1)) 1 ** pos ten (cr (c$6)) 6 + Cx (&2 * inv (&3 * sqrt (&2))) %
         pos ten (cr (b$1)) 1 ** pos ten (cr (d$5)) 5 +  Cx (sqrt (&2) * inv (&3)) %
         pos ten (cr (b$1)) 1 ** pos ten (cr (c$3)) 3) (tensor 6 (lambda j. vac (b$1)))`,	
			
	   ALL_CNOT_TAC);;


let CNOT_C1_T0 = prove
 (`!a b c d (ten:qop^N->(real^N->complex)-> (real^N->complex)).
	    is_tensor ten /\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,a$2,2,a$1,1,b$2,2,b$1,1)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,a$4,4,a$5,5,b$4,4,b$5,5)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$4,4,a$3,3,c$4,4,c$3,3)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$5,5,a$6,6,c$5,5,c$6,6)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,c$4,4,c$5,5,d$4,4,d$5,5)
         /\ vac (c$3)	= vac (b$1)   /\ vac (c$6) = vac(b$1)
	 ==>
	   (tensor 6 ((lambda i. if i = 3 then fock (a$3) 1 else (if i = 4 then fock (a$4) 1 else vac (a$2))):bqs^N)) = 
      (Cx (inv (&3)) %
       pos ten (cr (d$4)) 4 ** pos ten (cr (d$4)) 4 + Cx (inv (&3)) %
       pos ten (cr (d$4)) 4 ** pos ten (cr (c$6)) 6 + Cx (inv (&3)) %
       pos ten (cr (d$4)) 4 ** pos ten (cr (c$3)) 3 + Cx (inv (&3)) %
       pos ten (cr (d$5)) 5 ** pos ten (cr (d$4)) 4 + Cx (inv (&3)) %
       pos ten (cr (d$5)) 5 ** pos ten (cr (c$6)) 6 + Cx (inv (&3)) %
       pos ten (cr (d$5)) 5 ** pos ten (cr (c$3)) 3 + (--Cx (inv (&6)) - Cx (inv (&6))) %
	   pos ten (cr (c$3)) 3 ** pos ten (cr (d$4)) 4 + --Cx (inv (&3)) %
       pos ten (cr (c$3)) 3 ** pos ten (cr (c$6)) 6 + --Cx (inv (&3))%
       pos ten (cr (c$3)) 3 ** pos ten (cr (c$3)) 3)
     (tensor 6 (lambda j.  vac (b$1)))`,
	ALL_CNOT_TAC);;
     
let CNOT_C1_T1 = prove
 (`!a b c d (ten:qop^N->(real^N->complex)-> (real^N->complex)).
	    is_tensor ten /\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,a$2,2,a$1,1,b$2,2,b$1,1)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,a$4,4,a$5,5,b$4,4,b$5,5)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$4,4,a$3,3,c$4,4,c$3,3)/\
	   is_beam_splitter (Cx(sqrt(&1/ &3)),Cx(sqrt(&2/ &3)),Cx(sqrt(&2/ &3)),--Cx(sqrt(&1/ &3))
	   ,ten,b$5,5,a$6,6,c$5,5,c$6,6)/\
	   is_beam_splitter (Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),Cx(sqrt(&1/ &2)),--Cx(sqrt(&1/ &2))
	   ,ten,c$4,4,c$5,5,d$4,4,d$5,5)
         /\ vac (c$3)	= vac (b$1)   /\ vac (c$6) = vac(b$1)
	 ==>     
         (tensor 6 ((lambda i. if i = 3 then fock (a$3) 1 else (if i = 5 then fock (a$5) 1 else vac (a$2))):bqs^N)) = 
     (--Cx (inv (&3))  %
     pos ten (cr (d$4)) 4 ** pos ten (cr (c$6)) 6 +  Cx (inv (&3)) %
     pos ten (cr (d$4)) 4 ** pos ten (cr (d$5)) 5 + Cx (inv (&3)) %
     pos ten (cr (d$4)) 4 ** pos ten (cr (c$3)) 3 + --Cx (inv (&3)) %
     pos ten (cr (d$5)) 5 ** pos ten (cr (c$6)) 6 + Cx (inv (&3)) %
     pos ten (cr (d$5)) 5 ** pos ten (cr (d$5)) 5 + Cx (inv (&3)) %
     pos ten (cr (d$5)) 5 ** pos ten (cr (c$3)) 3 + Cx (inv (&3)) %
     pos ten (cr (c$3)) 3 ** pos ten (cr (c$6)) 6 + (--Cx (inv (&6)) - Cx (inv (&6)))%
	 pos ten (cr (c$3)) 3 ** pos ten (cr (d$5)) 5 + --Cx (inv (&3)) %
     pos ten (cr (c$3)) 3 ** pos ten (cr (c$3)) 3) 
     (tensor 6 (lambda j. vac (b$1)))`,
     ALL_CNOT_TAC);;